/* file_list.c : Routines for generating and managing
 *                lists of file in file system
 * David Porter
 * Minnesota Supercomputing Institute
 * University of Minnesota
 * Feb 2010
 */

#ifdef _WIN32
#include <windows.h>
#include <process.h>
#include <io.h>
#include <winbase.h>
typedef __int64 Offset_t;
#else
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
typedef long long Offset_t;
#endif

#include <stdio.h>
#include <string.h>
#include <malloc.h>
#include "glob.h"

/*
          typedef struct
          {
                  size_t gl_pathc;    \\ Count of paths matched so far
                  char **gl_pathv;    \\ List of matched pathnames.
                  size_t gl_offs;     \\ Slots to reserve in ‘gl_pathv’.
          } glob_t;
*/

/* Prototypes */
int file_shift(char *file, int nshift);
int ith_file_in_list(char *file);
int gen_file_list(char *filter);
int gen_filter(char *file, char *filter);
int get_ith_file(int iFile, char *cFile);
int ibegin_4_numerals(char *file);

/* static info */
int nfiles;
char cFiles[10000][256];

int get_file_index(char *cFile)
{
  int ibegin, index;
  ibegin = ibegin_4_numerals(cFile);
  if(ibegin < 0) return -1;
  if(sscanf(&cFile[ibegin], "%4d", &index) != 1) return -1;
  return index;
}

int get_ith_file(int iFile, char *cFile)
{
  if(iFile < 0 || iFile >= nfiles) return -1;
  sprintf(cFile, "%s", cFiles[iFile]);
  return 0;
}

int file_shift(char *file, int nshift)
{
  int ierr, ioldfile, inewfile;
  char filter[256];

  /* Generate file list if necessary and find file in list */
  ierr = gen_filter(file, filter);
  ierr = gen_file_list(filter);
  ioldfile = ith_file_in_list(file);

  /* return with no shift if file is not found in list */
  if(ioldfile < 0) { return 0; }

  /* shift file by nshift in list */
  inewfile = ioldfile + nshift;
  if(inewfile <       0) { inewfile = 0; }
  if(inewfile >= nfiles) { inewfile = nfiles - 1; }
  sprintf(file, "%s", cFiles[inewfile]);

  /* return size of shift */
  return inewfile-ioldfile;
}

int ith_file_in_list(char *file)
{
  int ifile, i;
  ifile = -1;
  for(i=0; i<nfiles; i++) { if(strcmp(cFiles[i], file) == 0) { ifile = i; } }
  return ifile;
}

int gen_file_list(char *filter)
{
  glob_t globbuf;
  int i, ierr;
  static char cCurrentFilter[256] = "";

  /* Reset current filter and file list */
  if(strcmp(filter, "") == 0) {
    sprintf(cCurrentFilter, "");
    nfiles = 0;
    return nfiles;
  }

  /* No need to redo file list if filter has not changed */
  if(strcmp(cCurrentFilter, filter) == 0) return nfiles;
  sprintf(cCurrentFilter, "%s", filter);

  /* Generate new list of files based on new filter */
  ierr = glob(filter, 0, NULL, &globbuf);
  nfiles = 0;
  if(ierr != 0) { return 0; }
  nfiles = (int)globbuf.gl_pathc;
  if(nfiles > 10000) nfiles = 10000;
  for(i=0; i<nfiles; i++) sprintf(cFiles[i], "%s", globbuf.gl_pathv[i]);
  globfree(&globbuf);

  return nfiles;
}

int ibegin_4_numerals(char *file)
{
  int len, i, i0, i9, ibegin;
  len = strlen(file);
  ibegin = -1;
  i0 = (int)'0';
  i9 = (int)'9';
  for(i=0; i<len-4; i++) {
    if(i0 <= (int)file[i  ]  &&  (int)file[i  ] <= i9  &&
       i0 <= (int)file[i+1]  &&  (int)file[i+1] <= i9  &&
       i0 <= (int)file[i+2]  &&  (int)file[i+2] <= i9  &&
       i0 <= (int)file[i+3]  &&  (int)file[i+3] <= i9    ) ibegin = i; 
  }
  return ibegin;
}

int gen_filter(char *file, char *filter)
{
  int ibegin;

  /* Assume: seguence number is 1st string of 4 numerals */
  ibegin = ibegin_4_numerals(file);
  if(ibegin < 0) return -1;

  /* Found a string of 4 numerals, now generate file filter for sequence */
  sprintf(filter, "%s", file);
  filter[ibegin  ] = '?';
  filter[ibegin+1] = '?';
  filter[ibegin+2] = '?';
  filter[ibegin+3] = '?';
  return 0;
}
